'' --------------------------------------------------------------------------
'' FreeBasic Wiki Editor
'' --------------------------------------------------------------------------

#inclib "curl"
#inclib "pcre"
#inclib "fbdoc"

#include once "windows.bi"
#include once "win/ole2.bi"

#include once "fbdoc_defs.bi"
#include once "CWikiCon.bi"
#include once "CWikiCache.bi"
#include once "fbdoc_cache.bi"
#include once "fbdoc_loader_web.bi"

#include once "resource.bi"

#include once "mkwiki.bi"

#ifdef HAVE_ASPELL
#include once "aspell.bi"
#endif

declare function MyWinMain ( byval hInstance as HINSTANCE, _
                   byval hPrevInstance as HINSTANCE, _
                   byref szCmdLine as string, _
                   byval iCmdShow as integer ) as integer    

using fb
using fbdoc
using fb.fbdoc

const web_wiki_url = "http://www.freebasic.net/wiki/wikka.php"
const dev_wiki_url = "http://myweb:8080/wiki/wikka.php"

'' Globals
extern as CWikiCon ptr wikicon 

dim shared as CWikiCon ptr wikicon = NULL
dim shared as string wiki_url
dim shared as string sCacheDir

dim shared as string appName
dim shared as string appTitle


#ifdef HAVE_ASPELL

extern as AspellConfig ptr spell_config
extern as AspellCanHaveError ptr possible_err
extern as AspellSpeller ptr spell_checker

dim shared as AspellConfig ptr spell_config = NULL
dim shared as AspellCanHaveError ptr possible_err = NULL
dim shared as AspellSpeller ptr spell_checker = NULL

#endif

'' --------------------------------------------------------------------------
'' MAIN ENTRY POINT
'' --------------------------------------------------------------------------

	dim result as integer

  	appName = "wmkwiki"
	appTitle = "fb wiki editor"
	if command(1) = "dev" then
		wiki_url = dev_wiki_url
	else
		wiki_url = web_wiki_url
	end if
	sCacheDir = curdir + "/cache/"

	? "Initializing ..."

	'' Initialize the connection (logon will happen on first post)
	wikicon = new CWikiCon( wiki_url )
	if wikicon = NULL then
		? "Unable to create connection " + wiki_url
		end 1
	end if

	'' Initialize the cache used by the doc converter / preview handler
	if LocalCache_Create( sCacheDir, CWikiCache.CACHE_REFRESH_NONE ) = FALSE then
		delete wikicon
		? "Unable to use local cache dir " + sCacheDir
		end 1
	end if

#ifdef HAVE_ASPELL

	'' Intitialize the spell checker
	spell_config = new_aspell_config()
	aspell_config_replace(spell_config, "lang", "en_US")
	possible_err = new_aspell_speller(spell_config)
	if (aspell_error_number(possible_err) <> 0) then
		delete wikicon
		LocalCache_Destroy()
		print aspell_error_message(possible_err)
		end 1
	end if

	spell_checker = to_aspell_speller(possible_err)
#endif

	RefreshPageIndex( FALSE )

	HtmlPreview_Init( curdir + "/" )

	result = MyWinMain( GetModuleHandle( null ), null, Command$, SW_NORMAL )

#ifdef HAVE_ASPELL
	delete_aspell_speller(spell_checker)
#endif
	if( wikicon ) then
		delete wikicon
	end if

	LocalCache_Destroy()
	Connection_Destroy()

	end result


'':::::
public function MyWinMain ( byval hInstance as HINSTANCE, _
                   byval hPrevInstance as HINSTANCE, _
                   byref szCmdLine as string, _
                   byval iCmdShow as integer ) as integer    
     
	dim wMsg as MSG
	dim hWnd as HWND
	dim ret as DWORD
    
	function = 0

	if( OleInitialize( NULL ) <> S_OK ) then
		return 1
	end if

	hwnd = Create_Main_Window( hInstance, AppName, AppTitle )

	if (hwnd) then
		ShowWindow( hWnd, iCmdShow )
		UpdateWindow( hWnd )
	else
		MessageBox( null, "Unable to create main window: " & str(GetLasterror()),  appName, MB_ICONERROR )
		return 1
	end if
 
	dim hAccelTable as HACCEL
	hAccelTable = LoadAccelerators( hInstance, cast( LPCSTR, IDR_MAIN_ACCEL ) )

	MainRefreshIndex()

	do    
		ret = GetMessage( @wMsg, null, 0, 0 )
		if ( ret ) = 0 then
			exit do
		elseif ( ret = - 1) then
			'' Error occured
			exit do
		else
			''if( IsDialogMessage( wMsg.hwnd, @wMsg ) = FALSE ) then
				if ( TranslateAccelerator( hwnd, hAccelTable, @wMsg) ) = 0 then
					TranslateMessage( @wMsg )
					DispatchMessage( @wMsg )
				end if
			''end if
		end if
	loop
	
	if (ret = -1) then
		MessageBox( null, "Error: " & str(GetLasterror()),  appName, MB_ICONERROR )
	end if

	OleUninitialize( )

	function = wMsg.wParam

end function
